

from sqlalchemy import Column
from sqlalchemy import String
from sqlalchemy import Integer
from sqlalchemy import ForeignKey
from sqlalchemy.orm import relationship
from sqlalchemy.ext.declarative import declared_attr

from tendril.authz.roles.devices import FleetAgencyRoleSpec
from tendril.authz.roles.devices import FleetRoleSpec
from tendril.authz.roles.devices import DeviceRoleSpec
from tendril.authz.roles.devices import DeviceContentRoleSpec

from tendril.db.models.interests import InterestModel
from tendril.db.models.deviceconfig import DeviceConfigurationModel
from tendril.db.models.content import ContentModel

from tendril.utils import log
logger = log.get_logger(__name__, log.DEFAULT)


class FleetAgencyModel(InterestModel):
    type_name = "fleet_agency"
    role_spec = FleetAgencyRoleSpec()

    id = Column(Integer, ForeignKey("Interest.id"), primary_key=True)

    __mapper_args__ = {
        "polymorphic_identity": type_name,
    }


class FleetModel(InterestModel):
    type_name = "fleet"
    role_spec = FleetRoleSpec()

    id = Column(Integer, ForeignKey("Interest.id"), primary_key=True)

    __mapper_args__ = {
        "polymorphic_identity": type_name,
    }


class DeviceModel(InterestModel):
    type_name = "device"
    role_spec = DeviceRoleSpec()

    id = Column(Integer, ForeignKey("Interest.id"), primary_key=True)
    appname = Column(String(32), nullable=False)
    config_id = Column(Integer, ForeignKey(DeviceConfigurationModel.id))

    @declared_attr
    def config(cls):
        return relationship(DeviceConfigurationModel, back_populates='devices', lazy='selectin')

    __mapper_args__ = {
        "polymorphic_identity": type_name,
    }


class DeviceContentModel(InterestModel):
    type_name = "device_content"
    role_spec = DeviceContentRoleSpec()

    id = Column(Integer, ForeignKey("Interest.id"), primary_key=True)
    content_id = Column(Integer, ForeignKey("Content.id"))

    @declared_attr
    def content(cls):
        return relationship(ContentModel, back_populates='device_content', lazy='selectin')

    __mapper_args__ = {
        "polymorphic_identity": type_name,
    }
